/*
 * scout-robotd/host.c
 *
 *                             ** COPYRIGHT NOTICE **
 *
 * Copyright (c) 2000, Nomadic Technologies, Inc.
 *
 * The contents of this file is copyrighted by Nomadic Technologies, Inc, and
 * is protected by United States copyright laws. You may make as many copies of
 * the software as deemed necessary for the effective programming of the Nomad
 * series robots provided that all copyright and other proprietary notices are
 * reproduced in their entirety.
 *
 */

#include <sys/types.h>
#include <unistd.h>

#include "robotd.h"


static const unsigned char host_gs_string[] = { 1, 2, 0, 7, 0xa, 0x5c };


#include <stdio.h>

/* ReadN(): read exactly count bytes from fd into buf.  this function will
            block indefinitely if insufficient bytes are available. */
int HOST_ReadN(int fd, void *buf, size_t count)
{
  int sofar = 0, len;

  do
  {
    if ((len = read(fd, ((unsigned char*)buf) + sofar, count - sofar)) < 0)
    {
      return -1;
    }
    else if (len == 0)
    {
      return 0;
    }
    sofar += len;
  } while (sofar < count);

  return sofar;
}



/* read_and_forward(): given a destination and source file descriptor,
 * forward one packet in the given direction.  if the destination is -1,
 * simply drop the packet.  this function also intercepts talk requests,
 * substituting them with a GS, which will return an appropriate response.
 */
int HOST_Forward(int dst, int src)
{
  unsigned char buf[BUFLEN];
  struct msg_hdr *bufhdr = (struct msg_hdr*)buf;   /* treat buf as a msg_hdr */
  int len;
  fd_set set;

  FD_ZERO(&set);
  FD_SET(src, &set);
  select(src + 1, &set, NULL, NULL, NULL);

  if ((len = HOST_ReadN(src, buf, sizeof(struct msg_hdr))) <= 0)
  {
    return len;
  }
  len = HOST_ReadN(src, buf+len, bufhdr->len0 + (bufhdr->len1 << 8));
  if (len <= 0)
  {
    return len;
  }
  len += sizeof(struct msg_hdr);

  /* check for talk requests */
  if (TK_IsTkRequest(buf))
  {
    /* send a GS message to the robot so it will return a proper reply */
    TK_Talk(bufhdr);
    write(dst, host_gs_string, sizeof(host_gs_string));
  }
  else if (dst >= 0)    /* only forward if a valid dst is given */
  {
    if (write(dst, buf, len) <= 0)
    {
      return -1;
    }
  }
  return len;
}

