/*
 * scout-robotd/tk.c
 *
 * this file determines whether speech is available on the
 * scout and executes the tk command on behalf of the client. 
 *
 *                             ** COPYRIGHT NOTICE **
 *
 * Copyright (c) 2000, Nomadic Technologies, Inc.
 *
 * The contents of this file is copyrighted by Nomadic Technologies, Inc, and
 * is protected by United States copyright laws. You may make as many copies of
 * the software as deemed necessary for the effective programming of the Nomad
 * series robots provided that all copyright and other proprietary notices are
 * reproduced in their entirety.
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "robotd.h"

static const char *tk_command_str;

#if 0
char **speech_prog = NULL;
char *cmd_begin = NULL;
char *cmd_end   = NULL;

char *dfl_speech_prog[] = {"festival", "--pipe", NULL};
char dfl_cmd_begin[] = "(SayText \"";
char dfl_cmd_end[]   = "\")";
char dfl_empty[] = "";

/* function prototypes */
void setup_child(void);
int start_speech(void);
void *alloca( size_t size);

int pipefd[2]; 		/* in linux, fd[0] receives while fd[1] sends */
#endif




void TK_Init(const char *command_str)
{
  if (command_str != NULL)
  {
    tk_command_str = command_str;
  }
  else
  {
    tk_command_str = "runfestival %s";
  }
  return;
}



int TK_IsTkRequest(char buf[])
{
  return ((struct msg_hdr*)buf)->type == TK;
}



#if 0
/* setup_child(): only called by child process.  exec the speech synthesizer.
 *      any return indicates an error 
 */
void setup_child()
{
  dup2(pipefd[0], 0);	/* copy pipe-out to stdin of child */
  close(pipefd[0]);
  close(pipefd[1]);
  execvp(speech_prog[0], (char**)speech_prog);
  perror("execvp");
}


/* find out if software speech is available on the present system.
 * returns 1 if it is available and 0 otherwise.
 */
int start_speech()
{
  static enum { UNINIT = 0, PRESENT, ABSENT } state = UNINIT;

  /* if speech is already running, do nothing */
  if (state == PRESENT)
  {
    return 0;
  }

  tk_avail = PRESENT; /* assume that it works until proven otherwise*/

  /* use the default speech program is nothing else is specified */
  if (speech_prog == NULL)
  {
    speech_prog = dfl_speech_prog;
    if (cmd_begin == NULL)
    {
      cmd_begin = dfl_cmd_begin;
    }
    if (cmd_end == NULL)
    {
      cmd_end = dfl_cmd_end;
    }
  }
  else
  {
    if (cmd_begin == NULL)
    {
      cmd_begin = dfl_empty;
    }
    if (cmd_end == NULL)
    {
      cmd_end = dfl_empty;
    }
  }

  if (pipe(pipefd) < 0)
  {
    return 0;
  }

  if (fork() == 0)
  {
    /* the child thread */
    setup_child();
    /* if we get here, the speech program could not be started */
    printf("unable to start child\n");
    exit(1);	/* exit the child process */
  }
  else
  {
    /* the parent thread */
    close(pipefd[0]);
  }

  return (tk_avail == TK_PRESENT) ? 0 : -1;
}
#endif


/* sw_tk(): attempt to start the speech software and send the requested
       string to it */
int TK_Talk(struct msg_hdr *msg)
{
  pid_t pid;

#if 0
  unsigned short len;
  char eol = '\n';
  int ret;

  len = msg->len0 + (msg->len1 << 8);
  ret = write(pipefd[1], cmd_begin, strlen(cmd_begin));
  ret = write(pipefd[1], msg->data, strlen(msg->data));
  ret = write(pipefd[1], cmd_end, strlen(cmd_end));
  ret = write(pipefd[1], &eol, 1);
#endif

  /* fork and run the requested command */

  if ((pid = fork()) == 0)
  {
    char cmd_buf[240];
    char *argv[4];

#ifdef _GNU_SOURCE
    if (snprintf(cmd_buf, sizeof(cmd_buf), tk_command_str, msg->data) == -1)
    {
      fprintf(stderr, "%s: resultant tk command string too long\n",
	      robd_program_name);
      return -1;
    }
#else
    sprintf(cmd_buf, tk_command_str, msg->data);
#endif

#if 1
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = cmd_buf;
    argv[3] = NULL;
    execve("/bin/sh", argv, environ);
#else
    printf("/bin/sh -c %s\n", cmd_buf);
#endif
    
  }
  else if (pid == -1)
  {
    fprintf(stderr, "%s: error in fork() for tk command: %s\n",
	    robd_program_name, strerror(errno));
  }

  return 0;
}
